
    .cdecls C, list, "msp430.h" ; this allows us to use C headers

    .text                       ; locates code in 'text' section

	.global dc_filter16

; Parameters
p           .set		R12
result      .set		R12
x           .set 		R13
tmp_lo      .set		R14
tmp_hi      .set		R15

; A routine to filter away the DC content from an AC mains waveform
; signal. It does this my using a heavily damped integrator to estimate
; the DC level. The current DC level is then subtracted from the signal.

; One would like to estimate DC by something like:
;   z += ((((int32_t) x << 15) - z) >> 14);
;   return  x - (z >> 15);
; but this:
;   z += ((((int32_t) x << 16) - z) >> 14);
;   return  x - (z >> 16);
; is a bit faster, and the shift by 16 will never
; cause an overflow in this application. However,
; remember this is not a generic DC filter!

;int16_t dc_filter16(int32_t *p, int16_t x);
dc_filter16:
    push.w  R11
    mov.w   0(p),tmp_lo      ; subtract (sample << 16) from filter_state
    mov.w   2(p),tmp_hi
    sub.w   x,tmp_hi
    ;
    rla.w   tmp_lo           ; >> 14
    rlc.w   tmp_hi
    subc.w  R11,R11
    inv.w   R11
    rla.w   tmp_lo
    rlc.w   tmp_hi
    rlc.w   R11
    ;
    sub.w   tmp_hi,0(p)     ; + filter_state
    subc.w  R11,2(p)
    sub.w   2(p),x          ; sample - filter_state
    mov.w   x,result
    pop.w   R11

	.if $defined(__LARGE_DATA_MODEL__)
    retx
	.else
    ret
    .endif

	.global dc_filter16_init

dc_filter16_init:
    ; Put the 16 bit initial estimate where the 16 bit signal will sit when the filter is running
    mov.w   #0,0(p)
    mov.w   x,2(p)

	.if $defined(__LARGE_DATA_MODEL__)
    retx
	.else
    ret
    .endif

    .end
