
    .cdecls C, list, "msp430.h" ; this allows us to use C headers

    .text                       ; locates code in 'text' section

	.global dc_filter24

; Parameters
x_ext	    .set 		R15
z           .set		R11
p           .set		R12
x           .set		R13

result_lo   .set		R12
result_hi   .set		R13
x_lo        .set		R10
x_loo		.set		R13
x_hi        .set		R14


; A routine to filter away the DC content from an AC mains waveform
; signal. It does this my using a heavily damped single pole leaky integrator,
; with noise shaping, to estimate the DC level. The current DC level is then
; subtracted from the signal.
; The actual calculations are:
;   z += (((x << 16) - z) >> 14);
;   return  x - (z >> 16);
; where z is a 48 bit value, stored as a 3 element array of 16 bit integers.

;int32_t dc_filter24(int16_t p[3], int32_t x);
dc_filter24:
    push.w  R10
    push.w  R11
    mov.w   p,z
    mov.w   x_loo,x_lo
    ;
    mov.w   x_lo,result_lo
    mov.w   x_hi,result_hi
    sub.w   2(z),x_lo
    subc.w  4(z),x_hi
    ; Shift, so our final effective shift is 14 bits.
    rla.w   x_lo
    rlc.w   x_hi
    subc.w  x_ext,x_ext
    inv.w   x_ext
    rla.w   x_lo
    rlc.w   x_hi
    rlc.w   x_ext
    ;
    add.w   x_lo,0(z)
    addc.w  2(z),x_hi
    addc.w  4(z),x_ext
    ; Store the updated *z
    mov.w   x_hi,2(z)
    mov.w   x_ext,4(z)
    ;
    sub.w   x_hi,result_lo
    subc.w  x_ext,result_hi
    ;
    pop.w   R11
    pop.w   R10

	.if $defined(__LARGE_DATA_MODEL__)
    retx
	.else
    ret
    .endif

	.global dc_filter24_init

dc_filter24_init:
    mov.w   #0,0(p)
    mov.w   #0,2(p)
    mov.w   #0,4(p)
    mov.b   x,3(p)
    swpb    x
    sxt     x
    mov.w   x,4(p)

	.if $defined(__LARGE_DATA_MODEL__)
    retx
	.else
    ret
    .endif

    .end

